/*
 * EngineContext.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using Eagle._Attributes;
using Eagle._Components.Public;

#if DEBUGGER
using Eagle._Components.Public.Delegates;
#endif

using Eagle._Containers.Private;
using Eagle._Containers.Public;
using Eagle._Interfaces.Private;
using Eagle._Interfaces.Public;

namespace Eagle._Components.Private
{
    [ObjectId("54a4fa59-05e7-4c54-ad72-d5496758c582")]
    internal sealed class EngineContext : IEngineContext, IDisposable
    {
        #region Public Constructors
        public EngineContext(int threadId)
        {
            this.threadId = threadId;

            levels = 0;
            previousLevels = 0;
            parserLevels = 0;
            expressionLevels = 0;
            traceLevels = 0;

#if NOTIFY || NOTIFY_OBJECT
            notifyLevels = 0;
            notifyTypes = NotifyType.None;
            notifyFlags = NotifyFlags.None;
#endif

            policyLevels = 0;
            testLevels = 0;

            stackOverflow = false;

#if DEBUGGER
            debugger = null;
            interactiveLoopCallback = null;
#endif

#if PREVIOUS_RESULT
            previousResult = null;
#endif

            engineFlags = EngineFlags.None;

            returnCode = ReturnCode.Ok;

            errorLine = 0;
            errorCode = null;
            errorInfo = null;
            errorFrames = 0;

            scriptLocation = null;
            scriptLocations = new StackList<IScriptLocation>();

            previousProcessId = 0;

            arraySearches = new ArraySearchDictionary();

#if HISTORY
            history = new ClientDataList();
#endif
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region IThreadContext Members
        private int threadId;
        public int ThreadId
        {
            get { return threadId; }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region IEngineContext Members
        private int levels;
        public int Levels
        {
            get { CheckDisposed(); return levels; }
            set { CheckDisposed(); levels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int previousLevels;
        public int PreviousLevels
        {
            get { CheckDisposed(); return previousLevels; }
            set { CheckDisposed(); previousLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int parserLevels;
        public int ParserLevels
        {
            get { CheckDisposed(); return parserLevels; }
            set { CheckDisposed(); parserLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int expressionLevels;
        public int ExpressionLevels
        {
            get { CheckDisposed(); return expressionLevels; }
            set { CheckDisposed(); expressionLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int unknownLevels;
        public int UnknownLevels
        {
            get { CheckDisposed(); return unknownLevels; }
            set { CheckDisposed(); unknownLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int traceLevels;
        public int TraceLevels
        {
            get { CheckDisposed(); return traceLevels; }
            set { CheckDisposed(); traceLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

#if NOTIFY || NOTIFY_OBJECT
        private int notifyLevels;
        public int NotifyLevels
        {
            get { CheckDisposed(); return notifyLevels; }
            set { CheckDisposed(); notifyLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private NotifyType notifyTypes;
        public NotifyType NotifyTypes
        {
            get { CheckDisposed(); return notifyTypes; }
            set { CheckDisposed(); notifyTypes = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private NotifyFlags notifyFlags;
        public NotifyFlags NotifyFlags
        {
            get { CheckDisposed(); return notifyFlags; }
            set { CheckDisposed(); notifyFlags = value; }
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        private int policyLevels;
        public int PolicyLevels
        {
            get { CheckDisposed(); return policyLevels; }
            set { CheckDisposed(); policyLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int testLevels;
        public int TestLevels
        {
            get { CheckDisposed(); return testLevels; }
            set { CheckDisposed(); testLevels = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private bool stackOverflow;
        public bool StackOverflow
        {
            get { CheckDisposed(); return stackOverflow; }
            set { CheckDisposed(); stackOverflow = value; }
        }

        ///////////////////////////////////////////////////////////////////////

#if DEBUGGER
        private IDebugger debugger;
        public IDebugger Debugger
        {
            get { CheckDisposed(); return debugger; }
            set { CheckDisposed(); debugger = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private InteractiveLoopCallback interactiveLoopCallback;
        public InteractiveLoopCallback InteractiveLoopCallback
        {
            get { CheckDisposed(); return interactiveLoopCallback; }
            set { CheckDisposed(); interactiveLoopCallback = value; }
        }
#endif

        ///////////////////////////////////////////////////////////////////////

#if PREVIOUS_RESULT
        private Result previousResult;
        public Result PreviousResult
        {
            get { CheckDisposed(); return previousResult; }
            set { CheckDisposed(); previousResult = value; }
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        private EngineFlags engineFlags;
        public EngineFlags EngineFlags
        {
            get { CheckDisposed(); return engineFlags; }
            set { CheckDisposed(); engineFlags = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private ReturnCode returnCode;
        public ReturnCode ReturnCode
        {
            get { CheckDisposed(); return returnCode; }
            set { CheckDisposed(); returnCode = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int errorLine;
        public int ErrorLine
        {
            get { CheckDisposed(); return errorLine; }
            set { CheckDisposed(); errorLine = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string errorCode;
        public string ErrorCode
        {
            get { CheckDisposed(); return errorCode; }
            set { CheckDisposed(); errorCode = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private string errorInfo;
        public string ErrorInfo
        {
            get { CheckDisposed(); return errorInfo; }
            set { CheckDisposed(); errorInfo = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int errorFrames;
        public int ErrorFrames
        {
            get { CheckDisposed(); return errorFrames; }
            set { CheckDisposed(); errorFrames = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private IScriptLocation scriptLocation;
        public IScriptLocation ScriptLocation
        {
            get { CheckDisposed(); return scriptLocation; }
            set { CheckDisposed(); scriptLocation = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private StackList<IScriptLocation> scriptLocations;
        public StackList<IScriptLocation> ScriptLocations
        {
            get { CheckDisposed(); return scriptLocations; }
            set { CheckDisposed(); scriptLocations = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private int previousProcessId;
        public int PreviousProcessId
        {
            get { CheckDisposed(); return previousProcessId; }
            set { CheckDisposed(); previousProcessId = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private ArraySearchDictionary arraySearches;
        public ArraySearchDictionary ArraySearches
        {
            get { CheckDisposed(); return arraySearches; }
            set { CheckDisposed(); arraySearches = value; }
        }

        ///////////////////////////////////////////////////////////////////////

#if HISTORY
        private IHistoryFilter historyEngineFilter;
        public IHistoryFilter HistoryEngineFilter
        {
            get { CheckDisposed(); return historyEngineFilter; }
            set { CheckDisposed(); historyEngineFilter = value; }
        }

        ///////////////////////////////////////////////////////////////////////

        private ClientDataList history;
        public ClientDataList History
        {
            get { CheckDisposed(); return history; }
            set { CheckDisposed(); history = value; }
        }
#endif

        ///////////////////////////////////////////////////////////////////////

        public EngineFlags BeginExternalExecution()
        {
            CheckDisposed();

            levels++;

            return Engine.AddStackCheckFlags(ref engineFlags);
        }

        ///////////////////////////////////////////////////////////////////////

        public void EndExternalExecution(
            EngineFlags savedEngineFlags
            )
        {
            CheckDisposed();

            Engine.RemoveStackCheckFlags(savedEngineFlags, ref engineFlags);

            levels--;
        }

        ///////////////////////////////////////////////////////////////////////

        public int BeginNestedExecution()
        {
            CheckDisposed();

            int savedPreviousLevels = previousLevels;
            previousLevels = levels;

            return savedPreviousLevels;
        }

        ///////////////////////////////////////////////////////////////////////

        public void EndNestedExecution(
            int savedPreviousLevels
            )
        {
            CheckDisposed();

            previousLevels = savedPreviousLevels;
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region IDisposable "Pattern" Members
        private bool disposed;
        private void CheckDisposed() /* throw */
        {
#if THROW_ON_DISPOSED
            if (disposed && Engine.IsThrowOnDisposed(null, false))
                throw new ObjectDisposedException(typeof(EngineContext).Name);
#endif
        }

        ///////////////////////////////////////////////////////////////////////

        private /* protected virtual */ void Dispose(
            bool disposing
            )
        {
            if (!disposed)
            {
                if (disposing)
                {
                    ////////////////////////////////////
                    // dispose managed resources here...
                    ////////////////////////////////////

                    threadId = 0;

                    ///////////////////////////////////////////////////////////

                    levels = 0;
                    previousLevels = 0;
                    parserLevels = 0;
                    expressionLevels = 0;
                    unknownLevels = 0;
                    traceLevels = 0;

#if NOTIFY || NOTIFY_OBJECT
                    notifyLevels = 0;
                    notifyTypes = NotifyType.None;
                    notifyFlags = NotifyFlags.None;
#endif

                    policyLevels = 0;
                    testLevels = 0;
                    stackOverflow = false;

#if DEBUGGER
                    interactiveLoopCallback = null;

                    if (debugger != null)
                    {
                        IDisposable disposable = debugger as IDisposable;

                        if (disposable != null)
                        {
                            disposable.Dispose();
                            disposable = null;
                        }

                        debugger = null;
                    }
#endif

#if PREVIOUS_RESULT
                    previousResult = null;
#endif

                    engineFlags = EngineFlags.None;
                    returnCode = ReturnCode.Ok;
                    errorLine = 0;
                    errorCode = null;
                    errorInfo = null;
                    errorFrames = 0;
                    scriptLocation = null;

                    ///////////////////////////////////////////////////////////

                    if (scriptLocations != null)
                    {
                        scriptLocations.Clear();
                        scriptLocations = null;
                    }

                    ///////////////////////////////////////////////////////////

                    previousProcessId = 0;

                    ///////////////////////////////////////////////////////////

                    if (arraySearches != null)
                    {
                        arraySearches.Clear();
                        arraySearches = null;
                    }

                    ///////////////////////////////////////////////////////////

#if HISTORY
                    historyEngineFilter = null;

                    if (history != null)
                    {
                        history.Clear();
                        history = null;
                    }
#endif
                }

                //////////////////////////////////////
                // release unmanaged resources here...
                //////////////////////////////////////

                disposed = true;
            }
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region IDisposable Members
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Destructor
        ~EngineContext()
        {
            Dispose(false);
        }
        #endregion
    }
}
