###############################################################################
#
# client.tcl --
#
# Extensible Adaptable Generalized Logic Engine (Eagle)
#
# Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
#
# See the file "license.terms" for information on usage and redistribution of
# this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# RCS: @(#) $Id: $
#
###############################################################################

#
# NOTE: Always skip network connectivity and download tests.
#
set no(network) true

#
# BUGFIX: If we are running in Tcl, the isEagle proc will not be
#         present yet and we need it now; therefore, create it if
#         necessary.
#
if {[llength [info commands isEagle]] == 0} then {
  #
  # NOTE: This is the procedure that detects whether or not we are
  #       running in Eagle (otherwise, we are running in vanilla Tcl).
  #       This procedure must function correctly in both Tcl and Eagle
  #       and must return non-zero only when running in Eagle.
  #
  proc isEagle {} {
    return [expr {[info exists ::tcl_platform(engine)] && \
      [string compare -nocase eagle $::tcl_platform(engine)] == 0}]
  }
}

#
# NOTE: Add the test constraint that will allow us to run the
#       isolated client socket test.
#
if {[isEagle]} then {
  #
  # NOTE: This will be processed properly after the [source] of
  #       the test prologue file (below).
  #
  lappend argv -constraints client
} else {
  #
  # NOTE: Make sure the tcltest package is available.
  #
  package require tcltest

  #
  # NOTE: Add the constraint for the isolated socket client test.
  #
  tcltest::testConstraint client 1
}

source [file join [file dirname [info script]] prologue.eagle]

#
# NOTE: Prevent nested calls into the test prologue and epilogue
#       file from being processed.
#
set no(prologue.eagle) true
set no(epilogue.eagle) true

#
# NOTE: Source the test file we are interested in.  Based on the
#       constraints, only one test should actually be run.
#
source [file join [file dirname [info script]] socket.eagle]

unset no(epilogue.eagle)
unset no(prologue.eagle)
unset no(network)

if {[array size no] == 0} then {unset no}

source [file join [file dirname [info script]] epilogue.eagle]
