/*
 * Class7.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using Eagle._Attributes;
using Eagle._Components.Public;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Sample
{
    [ObjectId("bd2eb8ee-cf20-4435-9d89-fa2c7d596ed4")]
    internal static class Class7
    {
        #region Private Data
        //
        // NOTE: Used for making random policy decisions.
        //
        private static Random random;
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Policy
        /// <summary>
        ///   Determine if the command with the specified arguments and context
        ///   should be allowed to execute.  The "Approved" method of the
        ///   policy context object should be called if the command should be
        ///   allowed to execute; however, this does not guarantee that the
        ///   command will execute.  If any other active policies call the
        ///   "Denied" method of the policy context object, execution of the
        ///   command will be vetoed.
        /// </summary>
        ///
        /// <param name="interpreter">
        ///   The interpreter context we are executing in.
        /// </param>
        ///
        /// <param name="clientData">
        ///   The extra data supplied when this command was initially created,
        ///   if any.
        /// </param>
        ///
        /// <param name="arguments">
        ///   The list of arguments supplied to this command by the script
        ///   being evaluated.
        /// </param>
        ///
        /// <param name="result">
        ///   Upon success, this must contain the result of the command.
        ///   Upon failure, this must contain an appropriate error message.
        ///   If no result is explicitly set by the command, a default result
        ///   of null will be used by the engine.
        /// </param>
        ///
        /// <returns>
        ///   ReturnCode.Ok on success, ReturnCode.Error on failure.
        /// </returns>
        [MethodFlags(MethodFlags.CommandPolicy)]
        private static ReturnCode PolicyCallback( /* POLICY */
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            //
            // NOTE: We only want to make policy decisions for our own
            //       command(s); therefore, use the appropriate helper
            //       method provided by the interpreter.
            //
            IPolicyContext policyContext = null;
            bool match = false;

            if (Utility.ExtractPolicyContextAndCommand(
                    interpreter, clientData, typeof(Class2), 0,
                    ref policyContext, ref match, ref result) == ReturnCode.Ok)
            {
                if (match)
                {
                    //
                    // NOTE: Create a new random policy decision maker to
                    //       allow us to easily demonstrate the effects of
                    //       both the "approved" and "denied" decisions.
                    //
                    if (random == null)
                        random = new Random();

                    //
                    // NOTE: Get a random integer; even numbers result in
                    //       a policy approval and odd numbers result in
                    //       policy denial.
                    //
                    if ((random.Next() % 2) == 0)
                        policyContext.Approved("random number is even");
                    else
                        policyContext.Denied("random number is odd");
                }

                return ReturnCode.Ok;
            }

            return ReturnCode.Error;
        }
        #endregion
    }
}
